;**************************************************************************
;* FileName	: KEY4.INC
;* Date 	: 09.10.2002
;* File version : v1.02
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: 4 KEY handler
;*		  ==================================
;*		  KEY4_init
;*		  KEY4_get
;*		  KEY4_poll
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* #define		KEY4_1		PORTB,4		; in
;* #define		KEY4_2		PORTB,5		; in
;* #define		KEY4_3		PORTB,6		; in
;* #define		KEY4_4		PORTB,7		; in
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define		KEY4ASCII_1	'1'
;* #define		KEY4ASCII_2	'2'
;* #define		KEY4ASCII_3	'3'
;* #define		KEY4ASCII_4	'4'
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  31/01/2001 	Initial Release
;* 1.01   15/06/2001	Minor changes
;* 1.02   09/10/2002    reformatting sources
;*
;**************************************************************************

;***** DEFINES ************************************************************
	ifdef	INC_KEY4
#define		KEY4_bufferfull	KEY4_flags, 0
#define		KEY4_debounce	KEY4_flags, 1
#define		KEY4_waitrel	KEY4_flags, 2
	endif

;***** VARIABLES **********************************************************

	ifdef	INC_KEY4
		cblock
		  KEY4_flags		; flags
		  KEY4_buffer		; keybuffer
		  KEY4_tempbuffer	; temp. keybuffer
		endc
	endif


;***** FUNCTIONS **********************************************************
	ifdef	INC_KEY4


;***** KEY4_init **********************************************************
;* Initialize four key routines
;* Input : -	        		; 
;* Output: - 				; 
;* Clock : 12
;**************************************************************************
KEY4_init	bank1
		bsf	KEY4_1
		bsf	KEY4_2
		bsf	KEY4_3	
		bsf	KEY4_4	
		bank0
		clrf	KEY4_flags
		clrf	KEY4_buffer
		return


;***** KEY4_get ***********************************************************
;* return with ASCII-code of the last pressed key
;* Input : -             		 
;* Output: w <- ASCII code of key or 0, if key not pressed
;* Clock : 3-17
;**************************************************************************
KEY4_get	btfss	KEY4_bufferfull
		retlw	0

		bcf	KEY4_bufferfull
		bsf	KEY4_waitrel
		
		movlw	HIGH KEY4_table
		movwf	PCLATH

		movf	KEY4_buffer, w
		andlw	00000011b
		addlw	LOW KEY4_table		; set CARRY
		skipnc
		incf	PCLATH,f

		clrf	KEY4_buffer		; KEY4_buffer = -1
		decf	KEY4_buffer, f

		movwf	PCL

KEY4_table
	ifndef	KEY4ASCII_1
		retlw	01h
		retlw	02h
		retlw	03h
		retlw	04h
	else
		retlw	KEY4ASCII_1
		retlw	KEY4ASCII_2
		retlw	KEY4ASCII_3
		retlw	KEY4ASCII_4
	endif


;***** KEY4_poll **********************************************************
;* Polling the four keys (should be call in every 10-50mS)
;* Input : -             		; 
;* Output: - 				; 
;* Clock : 3-29
;**************************************************************************
KEY4_poll	btfsc	KEY4_bufferfull
		return
		
;--------------------------------------------------------------------------
; KEY4_tempbuffer <- id of pressed key (0-3 or -1 not pressed)
;--------------------------------------------------------------------------
		clrf	KEY4_tempbuffer
		decf	KEY4_tempbuffer,f
		btfss	KEY4_1
		goto	KEY4_read4
		btfss	KEY4_2
		goto	KEY4_read3
		btfss	KEY4_3
		goto	KEY4_read2
		btfss	KEY4_4
		goto	KEY4_read1
		goto	KEY4_readend		

KEY4_read1	incf	KEY4_tempbuffer,f
KEY4_read2	incf	KEY4_tempbuffer,f
KEY4_read3	incf	KEY4_tempbuffer,f
KEY4_read4	incf	KEY4_tempbuffer,f

;--------------------------------------------------------------------------

KEY4_readend	btfsc	KEY4_debounce
		goto	KEY4_debo

		btfsc	KEY4_waitrel
		goto	KEY4_waitr

		incf	KEY4_tempbuffer, w
		skipnz
		return	

		movf	KEY4_tempbuffer,w
		movwf	KEY4_buffer
		bsf	KEY4_debounce
		return

		
KEY4_waitr	incf	KEY4_tempbuffer, w	; if (KEY4_tempbuffer==-1) KEY4_debounce=1
		skipnz
		bsf	KEY4_debounce
		return


KEY4_debo	bcf	KEY4_debounce
		movf	KEY4_tempbuffer,w
		xorwf	KEY4_buffer,w
		skipz
		return

		btfss	KEY4_waitrel		; if (!KEY4_waitrel) KEY4_bufferfull=1
		bsf	KEY4_bufferfull

		bcf	KEY4_waitrel
		return



;**************************************************************************
	endif


